//
//  FacebookApiManager.m
//  FriendBrowser
//
//  Created by Vishal Kapur on 7/9/11.
//  Copyright 2011 maha::software. All rights reserved.
//

#import "FacebookApiManager.h"
#import "SynthesizeSingleton.h"

@implementation FacebookApiManager

SYNTHESIZE_SINGLETON_FOR_CLASS(FacebookApiManager)

@synthesize facebookApi = _facebookApi;

- (id) init
{
	if (self = [super init])
	{
		self.facebookApi = [[Facebook alloc] initWithAppId:kFacebookAppID];

		[self initFacebookApi:_facebookApi];
	}
	return self;
}

#pragma mark -
#pragma mark Public api

- (void) login
{
	if (![_facebookApi isSessionValid]) 
	{
		NSArray *permissions = [NSArray arrayWithObjects:
								@"user_location", 
								@"friends_location", 
								@"user_education_history",
								@"friends_education_history",
								@"user_work_history",
								@"friends_work_history",
								@"user_interests",
								@"friends_interests",
								@"user_relationships",
								@"friends_relationships",
								@"offline_access",
								nil];
		[_facebookApi authorize:permissions delegate:self];
	}
	else {
		[[NSNotificationCenter defaultCenter] postNotificationName:kFacebookApiReadyNotification object:nil];
	}

}

- (Facebook *)cloneFacebookApi
{
	Facebook *fbClone = [[Facebook alloc] initWithAppId:kFacebookAppID];
	[self initFacebookApi:fbClone];
	return [fbClone autorelease];
}

#pragma mark -
#pragma mark helpers

- (void) initFacebookApi:(Facebook *)fbApi
{
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
	if ([defaults objectForKey:kFacebookAccessTokenKey] && [defaults objectForKey:kFacebookExpirationKey]) 
	{
		fbApi.accessToken = [defaults objectForKey:kFacebookAccessTokenKey];
		fbApi.expirationDate = [defaults objectForKey:kFacebookExpirationKey];
	}	
}

#pragma mark -
#pragma mark FBSessionDelegate

/**
 * Called when the user has logged in successfully.
 */
- (void)fbDidLogin 
{
	trace();
	NSUserDefaults *defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:[_facebookApi accessToken] forKey:kFacebookAccessTokenKey];
    [defaults setObject:[_facebookApi expirationDate] forKey:kFacebookExpirationKey];
    [defaults synchronize];
	
	[[NSNotificationCenter defaultCenter] postNotificationName:kFacebookApiReadyNotification object:nil];
}

/**
 * Called when the user canceled the authorization dialog.
 */
- (void)fbDidNotLogin:(BOOL)cancelled 
{
	trace();
}

/**
 * Called when the request logout has succeeded.
 */
- (void)fbDidLogout 
{
	trace();
}



@end
