//
//  FilterViewController.m
//  FriendBrowser
//
//  Created by Vishal Kapur on 7/9/11.
//  Copyright 2011 maha::software. All rights reserved.
//

#import "FilterViewController.h"


@implementation FilterViewController


#pragma mark -
#pragma mark Initialization

@synthesize delegate;

- (id) init
{
	if (self = [super initWithStyle:UITableViewStyleGrouped])
	{
		self.title = @"Filters";
		_filterFields = [[NSMutableArray alloc] init];
		
		for (int i=0; i < kFilterTotalNumberOfRows; i++)
			[_filterFields addObject:[[UITextField alloc] initWithFrame:kFilterTextFieldRect]];																		 

	}
	return self;
}

#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];

	UIBarButtonItem *cancelItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemCancel target:self action:@selector(cancelButtonTapped:)];
	self.navigationItem.leftBarButtonItem = cancelItem;
	[cancelItem release];

	UIBarButtonItem *doneItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemDone target:self action:@selector(doneButtonTapped:)];
	self.navigationItem.rightBarButtonItem = doneItem;
	[doneItem release];
	
}

#pragma mark -
#pragma mark Button handlers

- (void) cancelButtonTapped:(id)sender
{
	trace();
	[self.parentViewController dismissModalViewControllerAnimated:YES];
}

- (void) doneButtonTapped:(id)sender
{
	trace();
	NSMutableArray *filterValues = [NSMutableArray array];
	for (UITextField *filterField in _filterFields)
		[filterValues addObject:(filterField.text == nil) ? @"" : filterField.text];
	
	[self.delegate didSaveNewFilters:filterValues];
	[self.parentViewController dismissModalViewControllerAnimated:YES];
}

#pragma mark -
#pragma mark Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return kFilterTotalNumberOfRows;
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    static NSString *CellIdentifier = @"FilterCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue1 reuseIdentifier:CellIdentifier] autorelease];
    }
    
	cell.textLabel.text = [self fieldLabelAtRow:indexPath.row];
//	CGSize locSize = [cell.textLabel.text sizeWithFont:cell.textLabel.font];
	
	UITextField *filterField = [self filterFieldAtRow:indexPath.row];
	[cell.contentView addSubview:filterField];
	
    return cell;
}

- (UITextField *) filterFieldAtRow:(NSInteger)row
{
	UITextField *filterField = [_filterFields objectAtIndex:row];
	
	switch (row) {
		case FilterLocationRow:
			filterField.placeholder = @"City or State";
			break;
		case FilterSchoolRow:
			filterField.placeholder = @"High school or College";			
			break;
		case FilterEmployerRow:
			filterField.placeholder = @"Name of workplace";
			break;
			/*
		case FilterInterestsRow:
			filterField.placeholder = @"Activity, movies, or music";
			break;
			 */
		case FilterRelationshipStatusRow:
			filterField.placeholder = @"Single, married, etc";
			break;
		default:
			break;
	}

	return filterField;
}

- (NSString *) fieldLabelAtRow:(NSInteger)row
{
	switch (row) {
		case FilterLocationRow:
			return @"Location";
		case FilterSchoolRow:
			return @"School";
		case FilterEmployerRow:
			return @"Employer";
			/*
		case FilterInterestsRow:
			return @"Interest";
			 */
		case FilterRelationshipStatusRow:
			return @"Status";
	}

	return nil;
}

- (void)dealloc {
	delegate = nil;
	[_filterFields release];
    [super dealloc];
}


@end

