//
//  MainBrowseViewController.m
//  FriendBrowser
//
//  Created by Vishal Kapur on 7/8/11.
//  Copyright 2011 maha::software. All rights reserved.
//

#import "MainBrowseViewController.h"
#import "Friend.h"
#import "MSImageLoadOperation.h"
#import "FetchFriendInfoOperation.h"
#import "MSInterfaceUtils.h"
#import "MSWebPageDisplayController.h"

@implementation MainBrowseViewController

@synthesize facebookApi = _facebookApi;

#pragma mark -
#pragma mark Initialization

- (id) init
{
    self = [super initWithStyle:UITableViewStylePlain];
    if (self) {
		self.title = @"Friend Browser";
		
		_fbApiManager = [FacebookApiManager sharedFacebookApiManager];
		_thumbLoadQueue = [[NSOperationQueue alloc] init];
		_isLoadingFriendList = NO;
		
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(thumbDidLoad:) name:kMSImageDidLoadNotification object:nil];		
		[[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(facebookApiReady:) name:kFacebookApiReadyNotification object:nil];				
		

		UIBarButtonItem *filterItem = [[UIBarButtonItem alloc] initWithTitle:@"Filter" style:UIBarButtonItemStyleBordered target:self action:@selector(filterButtonTapped:)];
		self.navigationItem.leftBarButtonItem = filterItem;
		[filterItem release];

		UIBarButtonItem *aboutItem = [[UIBarButtonItem alloc] initWithTitle:@"About" style:UIBarButtonItemStyleBordered target:self action:@selector(aboutButtonTapped:)];
		self.navigationItem.rightBarButtonItem = aboutItem;
		[aboutItem release];
    }
    return self;
}

#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad 
{
    [super viewDidLoad];
	
	// login to facebook
	[_fbApiManager login];
}

#pragma mark -
#pragma mark Button handlers

- (void) filterButtonTapped:(id)sender
{
	trace();
	FilterViewController *controller = [[FilterViewController alloc] init];
	controller.modalTransitionStyle = UIModalTransitionStyleFlipHorizontal;
	controller.delegate = self;
	UINavigationController *filterNav = [[UINavigationController alloc] initWithRootViewController:controller];
	[self presentModalViewController:filterNav animated:YES];
	
	[controller release];
	[filterNav release];
}

- (void) aboutButtonTapped:(id)sender
{
	trace();
	
}

#pragma mark -
#pragma mark helpers

- (NSMutableArray *)currentFriendList
{
	if (_filters == nil)
		return _friendList;
	else
		return _filteredFriendList;
}

- (void) refreshFilteredList
{
	[_filteredFriendList release];
	_filteredFriendList = [[NSMutableArray alloc] init];

	for (Friend *curFriend in _friendList)
	{
		BOOL isMatch = YES;
		DLog(@"*** processing %@", curFriend.name);
		
		for (int filterIdx=0; filterIdx < [_filters count]; filterIdx++)
		{
			NSString *filterStr = [[_filters objectAtIndex:filterIdx] lowercaseString];
			NSString *friendFilterValue = [[curFriend valueForFilter:filterIdx] lowercaseString];
			DLog(@"filterStr = %@, friendFilterValue = %@", filterStr, friendFilterValue);
			
			if (friendFilterValue == nil && ![filterStr isEqualToString:@""]) {
				DLog(@"%@ did not match %@", curFriend.name, filterStr);
				isMatch = NO;
				break;
			}
			else if (friendFilterValue != nil && ![filterStr isEqualToString:@""]) {
				if ([friendFilterValue rangeOfString:filterStr].location == NSNotFound) {
					DLog(@"%@ did not match %@", curFriend.name, filterStr);
					isMatch = NO;
					break;
				}
			}
		}
		
		if (isMatch) [_filteredFriendList addObject:curFriend];
	}
	
	[self.tableView reloadData];
}	

#pragma mark -
#pragma mark FilterViewDelegate

- (void) didSaveNewFilters:(NSArray *)filters
{
	trace();
	[_filters release];
	_filters = [filters retain];
	
	[self refreshFilteredList];
}

#pragma mark -
#pragma mark FBRequestDelegate

/*
- (void)request:(FBRequest *)request didReceiveResponse:(NSURLResponse *)response 
{
	trace();
//	DLog(@"responseText = %@", request.responseText);
}
 */

- (void)request:(FBRequest *)request didLoadRawResponse:(NSData *)data
{
//	DLog(@"responseText = %@", [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]);
}

- (void)request:(FBRequest *)request didFailWithError:(NSError *)error 
{
	trace();
	NSLog(@"facebook error = %@", [error description]);
	
	UIAlertView *failAlert = [[UIAlertView alloc] initWithTitle:@"Error connecting to facebook" message:@"There was an error when connecting to facebook.  Please check your network connection and try again by tapping Retry." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:@"Retry",nil];
	[failAlert show];
	[failAlert release];
}

- (void)request:(FBRequest *)request didLoad:(id)result 
{

	if ([result isKindOfClass:[NSArray class]]) 
	{
		// response from batch request
		SBJsonParser *parser = [[SBJsonParser alloc] init];
		for (NSDictionary *batchResult in result)
		{
			NSDictionary *friendDict = [parser objectWithString:[batchResult objectForKey:@"body"]];
//			NSLog(@"id=%@, name=%@", [friendDict objectForKey:@"id"], [friendDict objectForKey:@"name"]);
//			NSLog(@"location=%@", [[friendDict objectForKey:@"location"] objectForKey:@"name"]);
			
			Friend *friend = [[Friend alloc] init];
			friend.friendID = [friendDict objectForKey:@"id"];
			friend.name = [friendDict objectForKey:@"name"];
			friend.locationName = [[friendDict objectForKey:@"location"] objectForKey:@"name"];
			if ([friend.locationName isKindOfClass:[NSNull class]]) friend.locationName = nil;
			friend.schoolNames = [self extractSchoolNames:[friendDict objectForKey:@"education"]];
			friend.employerNames = [self extractEmployerNames:[friendDict objectForKey:@"work"]];
			friend.relationshipStatus = [friendDict objectForKey:@"relationship_status"];
			if ([friend.relationshipStatus isKindOfClass:[NSNull class]]) friend.relationshipStatus = nil;
			
			[_friendList addObject:friend];
			[friend release];
		}
		
		if ([_friendList count] == _totalFriendCount) {
			// done loading all friend data
			// sort it first
			[_friendList sortUsingSelector:@selector(compareWithFriend:)];
			
			// reload table
			_isLoadingFriendList = NO;
			[self.tableView reloadData];
		}
	}
	else if ([result objectForKey:@"id"]) 
	{
		// response from /me
		NSLog(@"id=%@, name=%@", [result objectForKey:@"id"], [result objectForKey:@"name"]);
		NSLog(@"location=%@", [[result objectForKey:@"location"] objectForKey:@"name"]);
		
		[_fbApiManager.facebookApi requestWithGraphPath:@"me/friends" andDelegate:self];	
	}
	else
	{
		// response from /me/friends
		[_friendList release];
		_friendList = [[NSMutableArray alloc] init];
		
//		Friend *friend;
		NSArray *dataArray = [result objectForKey:@"data"];
		_totalFriendCount = [dataArray count];
		NSMutableArray *batchRequestArray = [[NSMutableArray alloc] init];
		int batchCount = 0;
		
		for (NSDictionary *friendDict in dataArray)
		{
			// graph.facebook.com/friend_id
			[batchRequestArray addObject:[NSDictionary dictionaryWithObjectsAndKeys:
										  @"GET", @"method",
										  [friendDict objectForKey:@"id"], @"relative_url",
										  nil]];
			
			/*
			// graph.facebook.com/friend_id/activities
			[batchRequestArray addObject:[NSDictionary dictionaryWithObjectsAndKeys:
										  @"GET", @"method",
										  [NSString stringWithFormat:@"%@/activities", [friendDict objectForKey:@"id"]], @"relative_url",
										  nil]];
			 */
			batchCount++;

			if (batchCount == 20)
			{
				// ready for batch request
				[self launchFetchOperationWithParams:batchRequestArray];
				batchCount = 0;
				[batchRequestArray release];
				batchRequestArray = [[NSMutableArray alloc] init];
			}
		}
		
		if (batchCount > 0)
			[self launchFetchOperationWithParams:batchRequestArray];
	}
}

- (void) launchFetchOperationWithParams:(NSArray *)batchRequestArray
{
	SBJsonWriter *jsonWriter = [[SBJsonWriter alloc] init];

	NSMutableDictionary *batchParams = [NSMutableDictionary dictionaryWithObjectsAndKeys:[jsonWriter stringWithObject:batchRequestArray], @"batch", nil];
	Facebook *fbClone = [_fbApiManager cloneFacebookApi];
	[fbClone requestWithGraphPath:@"me" andParams:batchParams andHttpMethod:@"POST" andDelegate:self]; // note: "me" is a dummy here

	/*
	FetchFriendInfoOperation *friendFetchOp = [[FetchFriendInfoOperation alloc] initWithBatchRequest:batchParams];
	[_friendLoadQueue addOperation:friendFetchOp];
	[friendFetchOp release];
	*/
}

- (NSString *)extractSchoolNames:(NSArray *)educationArray
{
	if (educationArray == nil || educationArray == (id)[NSNull null]) 
		return nil;
	
	NSMutableString *result = [NSMutableString string];
	for (NSDictionary *educationDict in educationArray) {
		[result appendString:[[educationDict objectForKey:@"school"] objectForKey:@"name"]];
		[result appendString:@"::"];  // delimiter
	}
	
	return result;
}

- (NSString *)extractEmployerNames:(NSArray *)workArray
{
	if (workArray == nil || workArray == (id)[NSNull null]) 
		return nil;
	
	NSMutableString *result = [NSMutableString string];
	for (NSDictionary *workDict in workArray) {
		[result appendString:[[workDict objectForKey:@"employer"] objectForKey:@"name"]];
		[result appendString:@"::"];  // delimiter
	}
	
	return result;
}

#pragma mark -
#pragma mark UIAlerViewDelegate methods

- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex 
{
	if (buttonIndex == 1)
		[self facebookApiReady:nil];
}

#pragma mark -
#pragma mark Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	if (_isLoadingFriendList)
		return 4;
	else
		return [[self currentFriendList] count];
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	
    static NSString *CellIdentifier;
	if (_isLoadingFriendList)
		CellIdentifier = @"LoadingCell";
	else
		CellIdentifier = @"FriendBrowseCell";
    
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier] autorelease];
    }

	if (_isLoadingFriendList)
	{
		cell.accessoryType = UITableViewCellAccessoryNone;

		if (indexPath.row == 3)
		{
			UILabel *loadingLabel = [MSInterfaceUtils newLabelWithPrimaryColor:[UIColor grayColor] selectedColor:[UIColor grayColor] fontName:@"Helvetica" fontSize:16 bold:YES];
			loadingLabel.text = @"Loading...";
			[loadingLabel sizeToFit];
			loadingLabel.center = cell.contentView.center;
			
			UIActivityIndicatorView *spinner = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleGray];
			
			spinner.frame = CGRectMake(loadingLabel.frame.origin.x - kLoadingSpinnerSize - 7, 
									   cell.frame.size.height/2 - kLoadingSpinnerSize/2, 
									   kLoadingSpinnerSize, 
									   kLoadingSpinnerSize);
			
			[cell.contentView addSubview:loadingLabel];
			[cell.contentView addSubview:spinner];    
			
			[spinner startAnimating];
			[spinner release];
		}
	}
	else
	{
		cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;

		Friend *currFriend = [[self currentFriendList] objectAtIndex:indexPath.row];
		cell.textLabel.text = currFriend.name;
		cell.detailTextLabel.text = currFriend.locationName;
		
		if (currFriend.profilePic == nil)
		{
			// use default image and fire off thumb load request
			currFriend.profilePic = [UIImage imageNamed:@"posty_emptyhand_tp_noback_57x57.png"];
			MSImageLoadOperation *imageLoadOp = [[MSImageLoadOperation alloc] initWithUrl:[currFriend profilePicURL]];
			imageLoadOp.indexPath = indexPath;
			[_thumbLoadQueue addOperation:imageLoadOp];
			[imageLoadOp release];		
		}
		cell.imageView.image = currFriend.profilePic;
	}
	
    return cell;
}


#pragma mark -
#pragma mark Table view delegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath 
{
	Friend *selectedFriend = [[self currentFriendList] objectAtIndex:indexPath.row];
	NSURL *webURL = [NSURL URLWithString:[NSString stringWithFormat:@"http://m.facebook.com/profile.php?id=%@&v=info", selectedFriend.friendID]];
	[[UIApplication sharedApplication] openURL:webURL];
	
	/*
	MSWebPageDisplayController *webpageDisplayCtr = [[MSWebPageDisplayController alloc] initWithURL:webURL title:selectedFriend.name];
	[self.navigationController pushViewController:webpageDisplayCtr animated:YES];
	[webpageDisplayCtr release];
	 */

	/*
	//	NSURL *fanPageURL = [NSURL URLWithString:[NSString stringWithFormat:@"fb://profile/%@", selectedFriend.friendID]];
	
	if (![[UIApplication sharedApplication] openURL: webURL]) {
        //fanPageURL failed to open.  Open the website in Safari instead
        NSURL *webURL = [NSURL URLWithString:@"http://www.facebook.com/pages/Blackout-Labs/210227459693"];
        [[UIApplication sharedApplication] openURL: webURL];
	}
	 */
}

#pragma mark -
#pragma mark Notification observers

- (void) facebookApiReady:(NSNotification *)note
{
	trace();
	_isLoadingFriendList = YES;
	[self.tableView reloadData];
	
	[_fbApiManager.facebookApi requestWithGraphPath:@"me" andDelegate:self];
}

/// MSImageDidLoad notification observer
- (void)thumbDidLoad:(NSNotification *)notification 
{
	UIImage *loadedImage = [notification.userInfo objectForKey:@"loadedImage"];
	NSIndexPath *indexPath = [notification.userInfo objectForKey:@"indexPath"];
	
	// note: need to check if indexPath is within bounds because ... well we don't really for this app
//	if (indexPath.row < [[self currentFriendList] count]) {
	
	Friend *friendAtIndexPath = [[self currentFriendList] objectAtIndex:indexPath.row];
	friendAtIndexPath.profilePic = loadedImage;
	[self.tableView reloadRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationNone];
}


- (void)dealloc {
	[_friendList release];
	[_thumbLoadQueue release];
    [super dealloc];
}


@end

